def print_song():
    """
    This function prints the sorting hat song.

    Arguments:
    None
    """    
    print(f"Oh you may not think I'm pretty, But don't judge on what you see, I'll eat myself if you can find A smarter hat than me.\n" \
            f"You can keep your bowlers black, Your top hats sleek and tall, For I'm the Hogwarts Sorting Hat And I can cap them all.\n" \
            f"There's nothing hidden in your head The Sorting Hat can't see, So try me on and I will tell you Where you ought to be.\n" \
            f"You might belong in Gryffindor, Where dwell the brave at heart, Their daring, nerve, and chivalry Set Gryffindors apart;\n" \
            f"You might belong in Hufflepuff, Where they are just and loyal, Those patient Hufflepuffs are true And unafraid of toil;\n" \
            f"Or yet in wise old Ravenclaw, if you've a ready mind, Where those of wit and learning, Will always find their kind;\n" \
            f"Or perhaps in Slytherin You'll make your real friends, Those cunning folks use any means To achieve their ends.\n" \
            f"So put me on! Don't be afraid! And don't get in a flap! You're in safe hands (though I have none) For I'm a Thinking Cap!"
            )